#include "Constants.h"
#include "Io.h"
#include "Keyboard.h"
#include "Kempston.h"
#include "DataBus.h"
#include "ULABus.h"
#include "Sound.h"

static Uint8 _portFE;

static Uint8 _borderModifiedLastFrame;
static Uint8 _borderModifiedThisFrame;

void io_mark_frame_start() {
	_borderModifiedLastFrame = _borderModifiedThisFrame;
	_borderModifiedThisFrame = 0;
}

Uint8 io_is_border_modified() {
	return _borderModifiedLastFrame;
}

void io_start() {
	io_destroy();

	_borderModifiedLastFrame = 1;
	_borderModifiedThisFrame = 1;
}

void io_destroy() {
	_portFE = 0;
}

Uint8 io_read8_16bitaddr(Uint16 port) {
	Uint8 value = 0;
	// joystick takes precedence over keyboard
	if (kempston_is_supported_address(port)) {
		value = kempston_read8(port);
		data_bus_write8(value);
		return value;
	}
	else if (keyboard_is_supported_address(port)) {
		value = keyboard_read8(port);
		data_bus_write8(value);
		// lack of return intended
	}
	else if (ula_bus_is_supported_address(port)) {
		value = ula_bus_read8(port);
		return value;
	}

	Uint8 ear;
	switch (port & 0xFF)
	{
	case ULA_PORT_FE:
		// bits 5 and 7 are always 1
		value |= 0xA0;

		// bit 6 := ear
		ear = (_portFE & 0x10) << 2;	// ear is written to bit 4 and is read from bit 6
		value &= 0xBF;
		value |= ear;

		return value;
	default:
		return 0xFF;
	}
}

// a worker specifically for writing to port FE
void _io_perform_port_FE_write(Uint8 value) {
	Uint8 newBorderValue = value & 7;
	Uint8 previousBorderValue = _portFE & 7;
	if (previousBorderValue != newBorderValue) {

	}

	_portFE = value;
}

void io_write8_16bitaddr(Uint16 port, Uint8 value) {
	data_bus_write8(value);

	switch (port & 0x00FF) {
	case ULA_PORT_FE:
		_io_perform_port_FE_write(value);
	default:
		break;
	}
}

void io_ear_write1(Uint8 value) {
	if (value) {
		_io_perform_port_FE_write(_portFE | 0x10);
	}
	else {
		_io_perform_port_FE_write(_portFE & 0xEF);
	}
}

Uint8 io_read8_border_colour() {
	return _portFE & 7;		// bits 0, 1, 2
}

Uint8 io_is_speaker_active() {
	return (_portFE & 0x10) >> 4;	// bit 4
}
